@@ -249,7 +249,10 @@ class AnswerBox(object):
             else:
                 abxml.tag = 'stringresponse'	# change to stringresponse for now (FIXME)
                 tl = etree.Element('textline')
-                tl.set('size','80')
+                if 'size' in abargs: 
+                    tl.set('size',self.stripquotes(abargs['size']))
+                else:
+                    tl.set('size','80')
                 abxml.append(tl)
                 abxml.set('answer','unknown')
                 self.copy_attrib(abargs,'inline',tl)
@@ -261,9 +264,12 @@ class AnswerBox(object):
             if 'size' in abargs: tl.set('size',self.stripquotes(abargs['size']))
             abxml.append(tl)
             abxml.set('answer',self.stripquotes(abargs['expect']))
+            if 'options' in abargs: 
+                abxml.set('type', self.stripquotes(abargs['options']))
+            else:
+                abxml.set('type','')
             self.copy_attrib(abargs,'inline',tl)
             self.copy_attrib(abargs,'inline',abxml)
-            abxml.set('type','')
 
         elif abtype=='customresponse':
             self.require_args(['expect','cfn'])
@@ -570,3 +576,80 @@ def split_args_with_quoted_strings(command_line, checkfn=None):
     if arg != '':
         arg_list.append(arg)
     return arg_list
+
+#-----------------------------------------------------------------------------
+# unit tests using py.test
+
+def test_option1():
+    abox = AnswerBox('type="option" expect="int" options="noneType","int","float"')
+    xmlstr = abox.xmlstr
+    print xmlstr
+    assert('''<optioninput options="('noneType','int','float')" correct="int"/>''' in xmlstr)
+
+def test_string1():
+    abox = AnswerBox('type="string" expect="Michigan" size="20"')
+    xmlstr = abox.xmlstr
+    print xmlstr
+    assert('<textline size="20"/>' in xmlstr)
+    assert('<stringresponse answer="Michigan" type="">' in xmlstr)
+
+def test_string2():
+    abox = AnswerBox('type="string" expect="Michigan" size="20" options="ci regexp"')
+    xmlstr = abox.xmlstr
+    print xmlstr
+    assert('<stringresponse answer="Michigan" type="ci regexp">' in xmlstr)
+
+def test_numerical1():
+    abox = AnswerBox('''expect="3.14159" type="numerical" tolerance='0.01' inline=1''')
+    xmlstr = abox.xmlstr
+    print xmlstr
+    assert('<numericalresponse inline="1" answer="3.14159">' in xmlstr)
+    assert('<textline inline="1">' in xmlstr)
+    assert('<responseparam type="tolerance" default="0.01"/>' in xmlstr)
+
+def test_formula1():
+    abox = AnswerBox("""expect="(-b + sqrt(b^2-4*a*c))/(2*a)" type="formula" """ +
+                     """samples="a,b,c@1,16,1:3,20,3#50" size="60" tolerance='0.01' inline='1' """ +
+                     """math="1" feqin="1" """)
+    xmlstr = abox.xmlstr
+    print xmlstr
+    assert('<formularesponse inline="1" type="cs" samples="a,b,c@1,16,1:3,20,3#50" answer="(-b + sqrt(b^2-4*a*c))/(2*a)">' in xmlstr)
+    assert('<formulaequationinput size="60" inline="1" math="1">' in xmlstr)
+    assert('<responseparam type="tolerance" default="0.01"/>' in xmlstr)
+    
+def test_multichoice1():
+    abox = AnswerBox('type="multichoice"  expect="Python","C++" options="Cobol","Pascal","Python","C++","Clu","Forth"')
+    xmlstr = abox.xmlstr
+    print xmlstr
+    assert('<choiceresponse>' in xmlstr)
+    assert(abox.xml.tag=='choiceresponse')
+    assert(len(abox.xml.findall('.//choice'))==6)
+    assert('<choice correct="true" name="3">' in xmlstr)
+    assert('<choice correct="true" name="4">' in xmlstr)
+
+def test_custom1():
+    abox = AnswerBox('''expect=""
+      type="custom"
+      answers="1,9"
+      prompts="x = ","y = "
+      cfn="sumtest"
+      inline="1" ''')
+    xmlstr = abox.xmlstr
+    print xmlstr
+    assert('<customresponse cfn="sumtest" inline="1" expect="">' in xmlstr)
+    assert('<p style="display:inline">x =<textline correct_answer="1" inline="1"/></p>' in xmlstr)
+    assert('<p style="display:inline">y =<textline correct_answer="9" inline="1"/></p>' in xmlstr)
+    assert('<br/>' in xmlstr)
+
+def test_custom1():
+    abox = AnswerBox('''expect="" type="jsinput" cfn="test_findep" 
+      width="650"
+      height="555"
+      gradefn="getinput"
+      get_statefn="getstate"
+      set_statefn="setstate"
+      html_file="/static/html/ps3plot_btran1.html"''')
+    xmlstr = abox.xmlstr
+    print xmlstr
+    assert('<customresponse cfn="test_findep" expect="">' in xmlstr)
+    assert('<jsinput width="650" height="555" gradefn="getinput" get_statefn="getstate" set_statefn="setstate" html_file="/static/html/ps3plot_btran1.html"/>' in xmlstr)
